@@ -2,7 +2,7 @@
   
    Expect_network commands
 
-   Copyright (C) 2007, 2008, 2009, 2010 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Eloy Paris
    Copyright (C) 1987-2002 Don Libes <libes@cme.nist.gov>,
 	National Institute of Standards and Technology <http://www.nist.gov/>
 
@@ -25,8 +25,8 @@
 
 #include "includes.h"
 
-#include <epan/packet.h> /* Apparently only for CHAR_ASCII */
-#include <wiretap/libpcap.h> /* For wtap_pcap_encap_to_wtap_encap() */
+#include <epan/epan.h>
+#include <wiretap/pcap-encap.h> /* For wtap_pcap_encap_to_wtap_encap() */
 
 #include "util-tcl.h"
 
@@ -71,33 +71,21 @@ fill_in_framedata(frame_data *fdata, struct nexp_listener *l,
 		  const struct pcap_pkthdr *h)
 {
     int ll_type;
-
+    struct wtap_pkthdr phdr;
     nstime_t first_ts, prev_cap_ts; /* For "struct timeval" -> nstime_t
 				       temporary conversions */
 
-    fdata->next = NULL;
-    fdata->prev = NULL;
-    fdata->pfd = NULL;
-    fdata->num = l->nreads; /* Frame number is number of times we've read
-			       from this listener */
-    fdata->pkt_len = h->len;
-    fdata->cum_bytes  = 0; /* XXX - can I set this to 0? EP.- */
-    fdata->cap_len = h->caplen;
-    fdata->file_off = 0; /* XXX - can I set this to 0? EP.- */
-
+    phdr.len = h->len;
+    phdr.caplen = h->caplen;
     ll_type = l->type == LISTENER_LIVE
 	      ? l->_live.datalink_type : l->_pcap.datalink_type;
-    fdata->lnk_t = wtap_pcap_encap_to_wtap_encap(ll_type);
+    phdr.pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
+    phdr.ts.secs = h->ts.tv_sec;
+    phdr.ts.nsecs = h->ts.tv_usec*1000;
 
-    fdata->abs_ts.secs = h->ts.tv_sec;
-    fdata->abs_ts.nsecs = h->ts.tv_usec*1000;
-    fdata->flags.passed_dfilter = 0;
-    fdata->flags.encoding = CHAR_ASCII;
-    fdata->flags.visited = 0;
-    fdata->flags.marked = 0;
-    fdata->flags.ref_time = 0;
-    fdata->color_filter = NULL;
+    frame_data_init(fdata, l->nreads /* num */, &phdr, 0 /* offset */, 0 /* cum_bytes */);
 
+#if 0
     /*
      * If we don't have the timestamp of the first packet provided by the
      * listener it's because this is the first packet. Save the timestamp of
@@ -148,14 +136,7 @@ fill_in_framedata(frame_data *fdata, struct nexp_listener *l,
     fdata->del_dis_ts = fdata->del_cap_ts;
 
     l->prev_cap_ts = h->ts;
-}
-
-/* Free up all data attached to a "frame_data" structure. */
-static void
-clear_fdata(frame_data *fdata)
-{
-    if (fdata->pfd)
-	g_slist_free(fdata->pfd);
+#endif
 }
 
 /*
@@ -186,7 +167,7 @@ process_packet(u_char *user, const struct pcap_pkthdr *h, const u_char *bytes)
     hd->passed_rfilter = pkt_dissect_tcl(hd->interp, bytes, &fdata, rfcode,
 					 ns_name, 1);
 
-    clear_fdata(&fdata);
+    frame_data_cleanup(&fdata);
 
     /*
      * We rely on the current time in some operations (when reading packets
