@@ -32,8 +32,9 @@
 
 #include <fcntl.h>
 #include <glib.h>
-#include <epan/packet.h> /* Apparently only for CHAR_ASCII */
-#include <wiretap/libpcap.h> /* For wtap_pcap_encap_to_wtap_encap() */
+
+#include <epan/epan.h>
+#include <wiretap/pcap-encap.h> /* For wtap_pcap_encap_to_wtap_encap() */
 
 #include "nexp_speakers.h"
 #include "util-tcl.h"
@@ -261,23 +262,15 @@ error:
 static void
 fill_in_framedata(frame_data *fdata, const struct pcap_pkthdr *h, int ll_type)
 {
-    fdata->next = NULL;
-    fdata->prev = NULL;
-    fdata->pfd = NULL;
-    fdata->num = 0;
-    fdata->pkt_len = h->len;
-    fdata->cum_bytes  = 0; /* XXX - can I set this to 0? EP.- */
-    fdata->cap_len = h->caplen;
-    fdata->file_off = 0; /* XXX - can I set this to 0? EP.- */
-    fdata->lnk_t = wtap_pcap_encap_to_wtap_encap(ll_type);
-    fdata->abs_ts.secs = h->ts.tv_sec;
-    fdata->abs_ts.nsecs = h->ts.tv_usec*1000;
-    fdata->flags.passed_dfilter = 0;
-    fdata->flags.encoding = CHAR_ASCII;
-    fdata->flags.visited = 0;
-    fdata->flags.marked = 0;
-    fdata->flags.ref_time = 0;
-    fdata->color_filter = NULL;
+    struct wtap_pkthdr phdr;
+
+    phdr.len = h->len;
+    phdr.caplen = h->caplen;
+    phdr.pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
+    phdr.ts.secs = h->ts.tv_sec;
+    phdr.ts.nsecs = h->ts.tv_usec*1000;
+
+    frame_data_init(fdata, 0 /* num */, &phdr, 0 /* offset */, 0 /* cum_bytes */);
 
     nstime_set_unset(&fdata->rel_ts);
     nstime_set_unset(&fdata->del_cap_ts);
@@ -324,6 +317,8 @@ build_and_hash(Tcl_Interp *interp, struct sent_packet **sent_packets)
 
 	packets[n].hash = pkt_hash(interp, packets[n].packet.data, &fdata);
 
+	frame_data_cleanup(&fdata);
+
 #ifdef DEBUG
 	printf("sent pkt len = %u, packet data:\n", packets[n].packet.len);
 	dump(packets[n].packet.data, packets[n].packet.len);
@@ -418,7 +413,7 @@ static int
 NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 		      Tcl_Obj * const *objv)
 {
-    int retval;
+    int retval, cmd_retval = TCL_OK;
     int i, j, n, npackets = 0, nunanswered;
     struct sent_packet *sent_packets = NULL;
     fd_set rfds;
@@ -441,6 +436,7 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
     int isanswer;
     Tcl_ThreadId thread_id;
     struct thread_data td;
+    int thread_status, thread_created = 0;
 
     nlisteners = 0;
     timeout.tv_sec = 1; timeout.tv_usec = 0;
@@ -449,8 +445,10 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
     memset(&pcb, 0, sizeof(pcb) );
 
     retval = process_options(interp, argc, objv);
-    if (retval == -1)
-	goto error;
+    if (retval == -1) {
+	pcb_destroy(&pcb);
+	return TCL_ERROR;
+    }
 
     /*
      * Make sure we have a listener we can use to listen for answers.
@@ -461,7 +459,8 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 	if (!llist[0]) {
 	    nexp_error(interp, "Can't find a suitable listener! Use "
 			       "spawn_network to create one.");
-	    goto error;
+	    pcb_destroy(&pcb);
+	    return TCL_ERROR;
 	}
 
 	nlisteners++;
@@ -479,7 +478,8 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 	if (!pcb.speaker) {
 	    nexp_error(interp, "Can't find a suitable speaker! Use "
 			       "spawn_network to create one.");
-	    goto error;
+	    pcb_destroy(&pcb);
+	    return TCL_ERROR;
 	}
     }
 
@@ -532,12 +532,14 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 
 	retval = Tcl_CreateThread(&thread_id, send_main, &td,
 				  TCL_THREAD_STACK_DEFAULT,
-				  TCL_THREAD_NOFLAGS);
+				  TCL_THREAD_JOINABLE);
 	if (retval != TCL_OK) {
 	    nexp_error(interp, "Tcl_CreateThread() error");
 	    goto error;
 	}
 
+	thread_created++;
+
 	/*
 	 * Receive loop. We loop while we still have unanswered packets
 	 * and while we have not timed out.
@@ -702,6 +704,8 @@ bypass_select:
 
 		hash = pkt_hash(interp, pkt_data, &rcvd_fdata);
 
+		frame_data_cleanup(&rcvd_fdata);
+
 #ifdef DEBUG
 		printf("recv'ed pkt len = %u, packet data:\n", pkthdr.len);
 		dump(pkt_data, pkthdr.len);
@@ -790,6 +794,9 @@ bypass_select:
 		    isanswer = pkt_isanswer(interp, pkt_data, &rcvd_fdata,
 					    sent_packets[j].packet.data,
 					    &sent_fdata);
+		    
+		    frame_data_cleanup(&sent_fdata);
+
 		    if (!isanswer)
 			/*
 			 * Hash does match, but packet is not an answer;
@@ -883,6 +890,13 @@ bypass_select:
 	}
 
 	close(pfds[0]);
+
+	/*
+	 * Need to wait for the sending thread even though we know it has
+	 * finished to avoid a memory leak, per Tcl_JoinThread()'s
+	 * documentation.
+	 */
+	Tcl_JoinThread(thread_id, &thread_status);
     } /* Main send/expect loop */
 
     /*
@@ -950,10 +964,7 @@ bypass_select:
 	printf("Sent %d packets; received %d answers; %d unanswered\n",
 	       npackets, nanswers, npackets - nanswers);
 
-    /*
-     * Clean up.
-     */
-
+cleanup:
     pcb_destroy(&pcb);
 
     for (i = 0; i < npackets; i++) {
@@ -969,10 +980,20 @@ bypass_select:
 
     free(sent_packets);
 
-    return TCL_OK;
+    return cmd_retval;
 
 error:
-    return TCL_ERROR;
+    cmd_retval = TCL_ERROR;
+
+    if (thread_created)
+	/*
+	 * Wait for sending thread to finish. We need this because parameters for
+	 * the sending thread are stored in this function stack, so if we return
+	 * without waiting for the sending thread to finish then we're in trouble.
+	 */
+	Tcl_JoinThread(thread_id, &thread_status);
+
+    goto cleanup;
 }
 
 #else /* ifndef __APPLE__ */
@@ -1205,6 +1226,8 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 
 		hash = pkt_hash(interp, pkt_data, &rcvd_fdata);
 
+		frame_data_cleanup(&rcvd_fdata);
+
 #ifdef DEBUG
 		printf("recv'ed pkt len = %u, packet data:\n", pkthdr.len);
 		dump(pkt_data, pkthdr.len);
@@ -1293,6 +1316,9 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 		    isanswer = pkt_isanswer(interp, pkt_data, &rcvd_fdata,
 					    sent_packets[j].packet.data,
 					    &sent_fdata);
+
+		    frame_data_cleanup(&sent_fdata);
+
 		    if (!isanswer)
 			/*
 			 * Hash does match, but packet is not an answer;
