@@ -29,9 +29,9 @@
 
 #include "includes.h"
 
-#include <epan/packet.h> /* Apparently only for CHAR_ASCII */
-#include <wiretap/libpcap.h> /* For wtap_pcap_encap_to_wtap_encap() */
 #include <epan/prefs.h>
+#include <epan/column-utils.h>
+#include <wiretap/pcap-encap.h> /* For wtap_pcap_encap_to_wtap_encap() */
 
 #include "proto_hier_stats.h"
 
@@ -123,37 +123,21 @@ get_line_buf(size_t len)
 static void
 fill_in_framedata(frame_data *fdata, const struct pcap_pkthdr *h, int ll_type)
 {
-    fdata->next = NULL;
-    fdata->prev = NULL;
-    fdata->pfd = NULL;
-    fdata->num = 0;
-    fdata->pkt_len = h->len;
-    fdata->cum_bytes  = 0; /* XXX - can I set this to 0? EP.- */
-    fdata->cap_len = h->caplen;
-    fdata->file_off = 0; /* XXX - can I set this to 0? EP.- */
-    fdata->lnk_t = wtap_pcap_encap_to_wtap_encap(ll_type);
-    fdata->abs_ts.secs = h->ts.tv_sec;
-    fdata->abs_ts.nsecs = h->ts.tv_usec*1000;
-    fdata->flags.passed_dfilter = 0;
-    fdata->flags.encoding = CHAR_ASCII;
-    fdata->flags.visited = 0;
-    fdata->flags.marked = 0;
-    fdata->flags.ref_time = 0;
-    fdata->color_filter = NULL;
+    struct wtap_pkthdr phdr;
+
+    phdr.len = h->len;
+    phdr.caplen = h->caplen;
+    phdr.pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
+    phdr.ts.secs = h->ts.tv_sec;
+    phdr.ts.nsecs = h->ts.tv_usec*1000;
+
+    frame_data_init(fdata, 0 /* num */, &phdr, 0 /* offset */, 0 /* cum_bytes */);
 
     nstime_set_unset(&fdata->rel_ts);
     nstime_set_unset(&fdata->del_cap_ts);
     nstime_set_unset(&fdata->del_dis_ts);
 }
 
-/* Free up all data attached to a "frame_data" structure. */
-static void
-clear_fdata(frame_data *fdata)
-{
-    if (fdata->pfd)
-	g_slist_free(fdata->pfd);
-}
-
 static
 char *columns_to_string(column_info *cinfo)
 {
@@ -382,7 +366,7 @@ update_string(Tcl_Obj *packet_obj)
 
     epan_dissect_run(edt, &pseudo_header, packet->bytes.data, &fdata, &cinfo);
 
-    epan_dissect_fill_in_columns(edt);
+    epan_dissect_fill_in_columns(edt, FALSE, TRUE);
 
     string_rep = columns_to_string(&cinfo);
 
@@ -397,7 +381,7 @@ update_string(Tcl_Obj *packet_obj)
     packet_obj->length = len;
 
     epan_dissect_free(edt);
-    clear_fdata(&fdata);
+    frame_data_cleanup(&fdata);
 }
 
 static void
@@ -521,7 +505,7 @@ tcl_packet_dissect(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 
     pkt_dissect_tcl(interp, packet->bytes.data, &fdata, NULL, ns_name, 1);
 
-    clear_fdata(&fdata);
+    frame_data_cleanup(&fdata);
 
     return TCL_OK;
 }
@@ -802,7 +786,7 @@ tcl_packet_show(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 
     pkt_dissect_show(packet->bytes.data, &fdata, verbose);
 
-    clear_fdata(&fdata);
+    frame_data_cleanup(&fdata);
 
     return TCL_OK;
 }
@@ -999,6 +983,7 @@ tcl_packet_new(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
     }
 
     if ( (pdu = pb_parsedef(pdudef, errbuf) ) == NULL) {
+	nexp_error(interp, "%s", errbuf);
 	free(pdudef);
 	return TCL_ERROR;
     }
